#include "gauge.h"
#include <QPaintEvent>
#include <gaugedrawable.h>
#include <qgridlayout.h>
#include <qpainter.h>
#include <math.h>
#include <qgroupbox.h>
#include <qcombobox.h>
#include <QLabel>
#include <QDoubleSpinBox>
#include <QTimer>
#include <qcheckbox.h>
#include <qscrollarea.h>
#include <QLineEdit>

void Frame::setDrawables(const QVector<GaugeDrawable *> &dras) {
    m_dras = dras;
}

void Frame::paintEvent(QPaintEvent *e) {
    QFrame::paintEvent(e);
    QPainter p(this);
    QPen epen(QColor("orange"));
    epen.setWidthF(3.0);
    p.setPen(QColor("orange"));
    p.drawRect(e->rect());
    p.setPen(QColor("violet"));
    p.drawRect(rect()); // frame rect

    foreach (GaugeDrawable * dra, m_dras) {
        dra->draw(&p, rect());
    }
}

Gauge::Gauge(QWidget *parent) : QWidget(parent) {
    m_cnt = 0.0;
    QHBoxLayout *hlo = new QHBoxLayout(this);
    Frame *frame = new Frame(this);
    hlo->addWidget(frame, 4);
    QVector<GaugeDrawable *>dras;

    // divide area in 4 squares
    GaugeDrawable *gd1 = new GaugeDrawable(this, palette());
    gd1->a.paint.center_rel = QPointF(0.5, 0.5);
    gd1->a.paint.size_rel = QSizeF(0.95, 0.95);
    gd1->a.bounds.min = 0.0;
    gd1->a.bounds.min_alarm = 35;
    gd1->a.bounds.min_warning = 70;
    gd1->a.bounds.max_warning = 140;
    gd1->a.bounds.max_alarm = 175;
    gd1->a.bounds.max = 210;
    gd1->setObjectName("g1");
    dras << gd1;
    frame->setDrawables(dras);

    QScrollArea *scrolla = new QScrollArea(this);
    int row = 0, col = 0;
    QGroupBox *confb = new QGroupBox("Config", this);
    QGridLayout *clo = new QGridLayout(confb);
    QLabel *sell = new QLabel("gauge select", this);
    QComboBox *cbsel = new QComboBox(this);
    cbsel->setObjectName("selector");
    for(int i = 1; i <= dras.size(); i++)
        cbsel->insertItem(cbsel->count(), QString::number(i));
    clo->addWidget(sell, row, col++, 1, 1);
    clo->addWidget(cbsel, row, col++, 1, 1);

    QCheckBox *cbupd = new QCheckBox("update", this);
    QSlider *sli = new QSlider(this);
    sli->setDisabled(true);
    sli->setOrientation(Qt::Horizontal);
    cbupd->setObjectName("cbupdate");
    cbupd->setChecked(true);
    clo->addWidget(cbupd, row, col++, 1, 1);
    clo->addWidget(sli, row, col++, 1, 1);
    connect(cbupd,SIGNAL(toggled(bool)), this, SLOT(onRefreshEnabled(bool)));

    foreach(const QString&l, QStringList()
                                   << "a.bounds.min"
                                   << "a.bounds.max" <<
                                   "a.bounds.min_warning"<<
                                   "a.bounds.max_warning"<<
                                   "a.bounds.min_alarm"<<
                                   "a.bounds.max_alarm" <<
                                       "a.paint.start" <<
                                       "a.paint.baseline"
                                   << "a.paint.span"
                                   << "a.paint.scale_alpha"
                                   << "a.paint.scale_pen_width"
                                             << "a.paint.width" <<
                                    "a.paint.scale_gauge_spacing"<<
                                   "a.paint.font_scale"
             ) {
        QLabel *la = new QLabel(l, this);
        QDoubleSpinBox *sb = new QDoubleSpinBox(this);
        sb->setObjectName(l);
        if(col % 2 == 0) {
            col = 0; row++;
        }
        sb->setMinimum(-1000);
        sb->setMaximum(1000);
        clo->addWidget(la, row, col++, 1, 1);
        clo->addWidget(sb, row, col++, 1, 1);
    }
    col = 0;
    foreach(const QString&l, QStringList () << "a.paint.show_range" << "a.paint.debug") {
        QCheckBox *cb = new QCheckBox(l, this);
        cb->setObjectName(l);
        clo->addWidget(cb, ++row, col, 1, 2);
        connect(cb, SIGNAL(toggled(bool)), this, SLOT(onCheckBoxToggled(bool)));
    }

    for(int i = 0; i < 1; i++) {
        QLineEdit *le = new QLineEdit(this);
        QLabel*l = new QLabel("r", this);
        le->setObjectName(QString("le%1").arg(i+1));
        QDoubleSpinBox *sbr = new QDoubleSpinBox(this);
        sbr->setObjectName("a.paint.internal_text.radius");
        sbr->setMinimum(0); sbr->setMaximum(1.0);
        sbr->setSingleStep(0.1);
        QLabel *la = new QLabel("angle", this);
        QDoubleSpinBox *sba = new QDoubleSpinBox(this);
        sba->setObjectName("a.paint.internal_text.angle");
        sba->setMinimum(0); sba->setMaximum(1.0);
        sba->setSingleStep(0.1);
        clo->addWidget(le, row, col++, 1, 1);
        clo->addWidget(l, row, col++, 1, 1);
        clo->addWidget(sbr, row, col++, 1, 1);
        clo->addWidget(la, row, col++, 1, 1);
        clo->addWidget(sba, row, col++, 1, 1);
        connect(le, SIGNAL(textChanged(QString)), this, SLOT(onLineEditTextChanged(QString)));
    }

    foreach(QDoubleSpinBox *sb, findChildren<QDoubleSpinBox *>())
        connect(sb, SIGNAL(valueChanged(double)), this, SLOT(onSpinValueChanged(double)));


    connect(cbsel, SIGNAL(currentTextChanged(QString)),
            this, SLOT(onCurrentSelChanged(QString)));
    // initialize spin boxes
    onCurrentSelChanged(cbsel->currentText());

    scrolla->setWidgetResizable(false);
    scrolla->setWidget(confb);
    hlo->addWidget(scrolla, 3);

    QTimer *t = new QTimer(this);
    t->setInterval(100);
    connect(t, SIGNAL(timeout()), this, SLOT(onTimeout()));
    t->start();
}

Gauge::~Gauge() {
}

void Gauge::onTimeout() {
    m_cnt++;
    foreach(GaugeDrawable *dra, findChildren<GaugeDrawable *>()) {
        if(dra->property("manual").toBool())
            continue;
        bool dec = dra->property("dec").toBool();
        float v = dra->value() + (dec ? -1 : 1);
        if(v <= dra->a.bounds.min) {
            dra->setProperty("dec", false); // increase
        }
        else if(v >= dra->a.bounds.max) {
            dra->setProperty("dec", true);
        }
        dra->setValue(v);
    }
    update();
}

void Gauge::onSpinValueChanged(double v) {
    const QString& g = "g" + findChild<QComboBox *>()->currentText();
    GaugeDrawable *ga = findChild<GaugeDrawable *>(g);
    const QString& par = sender()->objectName();
    if(par == "a.bounds.min")
        ga->a.bounds.min = v;
    else if(par == "a.bounds.max")
        ga->a.bounds.max = v;
    else if(par == "a.bounds.min_warning")
        ga->a.bounds.min_warning = v;
    else if(par == "a.bounds.max_warning")
        ga->a.bounds.max_warning = v;
    else if(par == "a.bounds.min_alarm")
        ga->a.bounds.min_alarm = v;
    else if(par == "a.bounds.max_alarm")
        ga->a.bounds.max_alarm = v;
    else if(par == "a.paint.start")
        ga->a.paint.start = v;
    else if(par == "a.paint.span")
        ga->a.paint.span = v;
    else if(par == "a.paint.baseline")
        ga->a.paint.baseline =  v;
    else if(par == "a.paint.scale_alpha")
        ga->a.paint.scale_alpha = v;
    else if(par ==  "a.paint.scale_pen_width")
        ga->a.paint.scale_pen_width = v;
    else if(par ==  "a.paint.width")
        ga->a.paint.width = v;
    else if(par ==  "a.paint.scale_gauge_spacing")
        ga->a.paint.scale_gauge_spacing = v;
    else if(par ==  "a.paint.font_scale")
        ga->a.paint.font_scale = v;


    else if(par ==  "a.paint.internal_text.radius")
        ga->a.paint.texts[0].radius = v;
    else if(par ==  "a.paint.internal_text.angle")
        ga->a.paint.texts[0].angle = v;

    findChild<Frame *>()->update();
}

void Gauge::onCheckBoxToggled(bool b) {
    const QString& g = "g" + findChild<QComboBox *>()->currentText();
    GaugeDrawable *ga = findChild<GaugeDrawable *>(g);
    const QString& par = sender()->objectName();
    if(par ==  "a.paint.show_range")
        ga->a.paint.show_range = b;
    else if(par ==  "a.paint.debug")
        ga->a.paint.debug = b;
    findChild<Frame *>()->update();
}

void Gauge::onCurrentSelChanged(const QString &t) {
    const QString& g = "g" + t;
    GaugeDrawable *ga = findChild<GaugeDrawable *>(g);
    double d;
    findChild<QDoubleSpinBox *>("a.bounds.min")
        ->setValue(ga->a.bounds.min);
    findChild<QDoubleSpinBox *>("a.bounds.max")
        ->setValue(ga->a.bounds.max);
    findChild<QDoubleSpinBox *>("a.bounds.min_warning")
        ->setValue(ga->a.bounds.min_warning);
    findChild<QDoubleSpinBox *>("a.bounds.max_warning")
        ->setValue(ga->a.bounds.max_warning);
    findChild<QDoubleSpinBox *>("a.bounds.min_alarm")
        ->setValue(ga->a.bounds.min_alarm);
    findChild<QDoubleSpinBox *>("a.bounds.max_alarm")
        ->setValue(ga->a.bounds.max_alarm);
    findChild<QDoubleSpinBox *>("a.paint.start")
        ->setValue(ga->a.paint.start);
    findChild<QDoubleSpinBox *>("a.paint.span")
        ->setValue(ga->a.paint.span);
    findChild<QDoubleSpinBox *>("a.paint.baseline")
        ->setValue(ga->a.paint.baseline);
    findChild<QDoubleSpinBox *>("a.paint.font_scale")
        ->setValue(ga->a.paint.font_scale);
    findChild<QDoubleSpinBox *>("a.paint.scale_alpha")
        ->setValue(ga->a.paint.scale_alpha);
    findChild<QDoubleSpinBox *>("a.paint.scale_pen_width")
        ->setValue(ga->a.paint.scale_pen_width);
    findChild<QDoubleSpinBox *>("a.paint.width")
        ->setValue(ga->a.paint.width);
    findChild<QDoubleSpinBox *>("a.paint.scale_gauge_spacing")
        ->setValue(ga->a.paint.scale_gauge_spacing);
    findChild<QCheckBox *>("a.paint.show_range")
        ->setChecked(ga->a.paint.show_range);
    findChild<QCheckBox *>("a.paint.debug")
        ->setChecked(ga->a.paint.debug);


    findChild<QDoubleSpinBox *>("a.paint.internal_text.angle")
        ->setValue(ga->a.paint.texts[0].angle);

    findChild<QDoubleSpinBox *>("a.paint.internal_text.radius")
        ->setValue(ga->a.paint.texts[0].radius);

}

void Gauge::onSliderChanged(int v) {
    const QString& g = "g" + findChild<QComboBox *>()->currentText();
    GaugeDrawable *ga = findChild<GaugeDrawable *>(g);
    ga->setValue(v);
}

// en true: timer refresh
void Gauge::onRefreshEnabled(bool en) {
    const QString& g = "g" + findChild<QComboBox *>()->currentText();
    GaugeDrawable *ga = findChild<GaugeDrawable *>(g);
    QSlider *sli = findChild<QSlider *>();
    sli->setEnabled(!en);
    ga->setProperty("manual", !en);
    if(!en) {
        sli->setRange(ga->a.bounds.min, ga->a.bounds.max);
        sli->setValue(ga->value());
        connect(sli, SIGNAL(valueChanged(int)), this, SLOT(onSliderChanged(int)));
    } else {
        disconnect(sli, SIGNAL(valueChanged(int)), this, SLOT(onSliderChanged(int)));
    }
}

void Gauge::onLineEditTextChanged(const QString &txt) {
    int t = sender()->objectName().remove("le").toInt();
    const QString& g = "g" + findChild<QComboBox *>()->currentText();
    GaugeDrawable *ga = findChild<GaugeDrawable *>(g);
    ga->a.paint.texts[0].s = txt;
}
