#ifndef GAUGEDRAWABLE_H
#define GAUGEDRAWABLE_H

#include <QObject>
#include <drawable_i.h>
#include <qfont.h>
#include <qupalette.h>

class GaugeDrawableP;

struct internal_text {
    QString s; //!< text to show, if not empty
    float radius = 0.7; //!< draw at distance radius * side / 2.0
    float angle = 0.75; //!< draw at angle * 360. 0 degrees at 3 o'clock. Default: 6 o clock. Counterclockwise
};

struct gauge_dra_paint {
    enum QualityColor { Normal = 0, Warning, Alarm, MaxQualityColor };
    bool error = false; //!< draw an error indication
    QFont font; //!< set a specific font. See font_scale below
    float font_scale = 1.0; //!< fontSize will be painter's fontSize*font_scale. Used only if font is not specified
    float scale_alpha = 180; //!< alarm / warning / normal color scale: alpha channel. default 180
    float scale_pen_width = 0.0f;
    float scale_gauge_spacing = 1.0f; //!< spacing between external scale and gauge
    QColor number, background;
    QColor gauge[MaxQualityColor] = { QColor("green"), QColor("orange"), QColor("red")}; //!< triplet with normal color, warning (default: orange) and alarm (default: red)
    bool show_number = true;
    bool show_range = true;
    QString number_format = "%.1f";
    QString bounds_number_format = "%.0f";
    float start = 210; //!< start drawing gauge: 210 degrees counterclockwise starting from 3 o'clock
    float  span = 240; //!< gauge span angle, in degrees
    float width = -1.0; //!< width of the gauge stripe: < 0 means auto
    QPointF center_rel = QPointF(0.5, 0.5); //!< use center of rect passed in draw by default
    QSizeF size_rel = QSizeF(1.0, 1.0); //!< use full rect size passed in draw by default

    float baseline = std::numeric_limits<float>::lowest();
    internal_text texts[4];

    bool debug = false; //!< activate some debug drawing (relevant areas and so on)
};
struct gauge_dra_bounds {
    float min = 0.0, max = 1000.0,
        min_alarm = 0.0, max_alarm = 0.0,
        min_warning = 0.0,  max_warning = 0.0;
};

struct gauge_dra_attributes {
    struct gauge_dra_paint paint;
    struct gauge_dra_bounds bounds;
};

class GaugeDrawable : public QObject, public Drawable_I
{
    Q_OBJECT
public:

    gauge_dra_attributes a;

    explicit GaugeDrawable(QObject *parent, const QPalette& palette);
    virtual ~GaugeDrawable ();
    float value() const;

public slots:
    void setValue(float v);

signals:

    // Drawable_I interface
public:
    bool scales();
    int z() const;
    QSizeF relativeSize() const;
    QPointF relativeCenter() const;
    void draw(QPainter *p, const QRectF &r);

private:
    GaugeDrawableP *d;

    void m_init_from(const QPalette& p);
};



#endif // GAUGEDRAWABLE_H
